"""release_manager: __init__.py declaration"""

import release_manager.targets
import release_manager.__main__
import release_manager._version
import release_manager.logger
import release_manager.package
import release_manager.utils
